@@ -1,6 +1,9 @@
 package URL::Normalize;
-use strict;
-use warnings;
+use Moose;
+use namespace::autoclean;
+
+use URI qw();
+use URI::QueryParam qw();
 
 =head1 NAME
 
@@ -8,82 +11,27 @@ URL::Normalize - Normalize/optimize URLs.
 
 =head1 VERSION
 
-Version 0.22
+Version 0.30
 
 =cut
 
-our $VERSION = '0.22';
-
-use URI qw();
-use URI::QueryParam qw();
+our $VERSION = '0.30';
 
 =head1 SYNOPSIS
 
     use URL::Normalize;
 
-    my $normalizer = URL::Normalize->new(
-        url => 'http://www.example.com/display?lang=en&article=fred',
-    );
+    my $normalizer = URL::Normalize->new( 'http://www.example.com/display?lang=en&article=fred' );
 
-    # ...or
+    # Normalize the URL of your choosing.
+    $normalizer->remove_social_query_params;
+    $normalizer->make_canonical;
 
-    my $normalizer = URL::Normalize->new(
-        url  => '/display?lang=en&article=fred',
-        base => 'http://www.example.com',
-    );
-
-    # Get a normalized URL back
-
-    $normalizer->do_all; # Perform all the normalizations available
-
-    print $normalizer->get_url; # or $normalizer->url;
+    # Get the normalized version back.
+    my $url = $normalizer->url;
 
 =cut
 
-sub new {
-    my $class = shift;
-
-    my $self = {
-        url  => undef,
-        base => undef,
-    };
-    bless( $self, $class );
-
-    $self->_init( @_ );
-
-    return $self;
-}
-
-sub _init {
-    my $self = shift;
-    my %opts = @_;
-
-    if ( defined $opts{url} && length $opts{url} ) {
-        $self->{url} = $opts{url};
-    }
-    else {
-        Carp::croak( "The 'url' parameter is required!" );
-    }
-
-    if ( defined $opts{base} && length $opts{base} ) {
-        $self->{base} = $opts{base};
-    }
-
-    my $URI = $self->get_URI;
-
-    unless ( $URI->scheme ) {
-        Carp::croak( "Illegal 'url' and/or 'base' values for '$URI'!" );
-    }
-
-    return 1;
-}
-
-sub DESTROY {
-    my $self = shift;
-
-    return 1;
-}
-
 =head1 DESCRIPTION
 
 This is NOT a perfect solution. If you normalize a URL using all the methods in
@@ -120,77 +68,112 @@ methods:
 
 =head1 CONSTRUCTORS
 
-=head2 new( %opts )
+=head2 new( $url )
 
-Constructs a new URL::Normalize object. Takes a hash as input argument;
+Constructs a new URL::Normalize object:
 
-    my $normalizer = URL::Normalize->new(
-        url  => '/foobar.html',            # required
-        base => 'http://www.example.com/', # optional
-    );
+    my $normalizer = URL::Normalize->new( 'http://www.example.com/some/path' );
+
+You can also send in just the path:
+
+    my $normalizer = URL::Normalize->new( '/some/path' );
+
+The latter is not recommended, however, so you should look into L<URI>'s
+C<new_abs> to create "real" URLs before you send them to URL::Normalize.
 
 =cut
 
 =head1 METHODS
 
-=head2 get_URI
+=cut
 
-Returns the C<URI> object representing the current state of the URL.
+around BUILDARGS => sub {
+    my $orig  = shift;
+    my $class = shift;
+
+    if ( @_ == 1 && !ref $_[0] ) {
+        return $class->$orig( url => $_[0] );
+    }
+    else {
+        return $class->$orig( @_ );
+    }
+};
+
+=head2 url
+
+Get the current URL, preferably after you have run one or more of the
+normalization methods.
 
 =cut
 
-sub get_URI {
-    my $self = shift;
+has 'url' => (
+    isa      => 'Str',
+    is       => 'rw',
+    required => 1,
+    writer   => '_set_url',
+);
 
-    return URI->new( $self->get_url, $self->get_base );
-}
+has 'dir_index_regexps' => (
+    isa     => 'ArrayRef[Str]',
+    is      => 'rw',
+    default => sub {
+        [
+            '/default\.aspx?',
+            '/index\.cgi',
+            '/index\.php\d?',
+            '/index\.pl',
+            '/index\.s?html?',
+        ];
+    },
+);
+
+has 'social_query_params' => (
+    isa     => 'ArrayRef[Str]',
+    is      => 'rw',
+    default => sub {
+        [
+            'ncid',
+            'utm_campaign',
+            'utm_medium',
+            'utm_source',
+        ],
+    },
+);
 
 =head2 get_url
 
-Returns the current URL.
+DEPRECATED! Use C<url> instead.
 
 =cut
 
-sub _get {
-    my $self = shift;
-    my $key  = shift;
-
-    return ( defined($self->{$key}) && length($self->{$key}) ) ? $self->{$key} : '';
-}
-
 sub get_url {
     my $self = shift;
 
-    return $self->_get( 'url' );
+    Carp::carp( "The 'get_url' method is deprecated; start using the 'url' method instead." );
+
+    return $self->url;
 }
 
-=head2 url
+=head2 URI
 
-Returns the current URL.
+Returns a L<URI> representation of the current URL.
 
 =cut
 
-sub url {
-    return shift->get_url;
-}
-
-sub _set_url {
-    my $self = shift;
-    my $url  = shift;
-
-    $self->{url} = $url;
-}
-
-=head2 get_base
-
-Returns the current base.
-
-=cut
-
-sub get_base {
+sub URI {
     my $self = shift;
 
-    return $self->_get( 'base' );
+    my $URI = undef;
+
+    eval {
+        $URI = URI->new( $self->url );
+    };
+
+    if ( $@ ) {
+        Carp::carp( "Failed to create a URI object from URL '" . $self->url . "'" );
+    }
+
+    return $URI;
 }
 
 =head2 make_canonical
@@ -213,32 +196,36 @@ following steps (at least):
 Example:
 
     my $normalizer = URL::Normalize->new(
-        url => 'http://www.example.com/%7Eusername/',
+        url => 'HTTP://www.example.com:80/%7Eusername/',
     );
 
-    $Normalize->make_canonical;
+    $normalizer->make_canonical;
 
-    print $Normalize->get_url; # http://www.example.com/~username/
+    print $normalizer->url; # http://www.example.com/~username/
 
 =cut
 
 sub make_canonical {
     my $self = shift;
 
-    #
-    # Set new 'url' value
-    #
-    return $self->_set_url( $self->get_URI->canonical->as_string );
+    if ( my $URI = $self->URI ) {
+        $self->_set_url( $URI->canonical->as_string );
+    }
+    else {
+        Carp::carp( "Can't make non-URI URLs canonical." );
+    }
 }
 
 =head2 remove_dot_segments
 
-The segments ".." and "." will be removed and "folded" (or flattened, if you
-prefer) from the URL.
+The C<.>, C<..> and C<...> segments will be removed and "folded" (or "flattened", if
+you prefer) from the URL.
 
-This method does NOT follow the algorithm described in RFC 3986, but rather
-flattens each path segment. It works just as well, it seems, but keep in mind
-that it also doesn't account for symbolic links on the server side.
+This method does NOT follow the algorithm described in L<RFC 3986: Uniform Resource Indentifier|http://tools.ietf.org/html/rfc3986>,
+but rather flattens each path segment.
+
+Also keep in mind that this method doesn't (can't) account for symbolic links
+on the server side.
 
 Example:
 
@@ -255,40 +242,40 @@ Example:
 sub remove_dot_segments {
     my $self = shift;
 
-    my $URI = URI->new( $self->get_url, $self->get_base );
+    my @old_path_segments = ();
+    my @new_path_segments = ();
 
-    my @old_segments = split( '/', $URI->path_segments );
-    my @new_segments = ();
+    my $URI = $self->URI;
 
-    foreach my $segment ( @old_segments ) {
-        if ( $segment eq '.' ) {
-            next;
+    if ( my $URI = $self->URI ) {
+        @old_path_segments = split( '/', $URI->path_segments );
+
+        foreach my $segment ( @old_path_segments ) {
+            if ( $segment eq '.' || $segment eq '...' ) {
+                next;
+            }
+
+            if ( $segment eq '..' ) {
+                pop( @new_path_segments );
+                next;
+            }
+
+            push( @new_path_segments, $segment );
         }
 
-        if ( $segment eq '..' ) {
-            pop( @new_segments );
-            next;
+        if ( @new_path_segments ) {
+            $URI->path_segments( @new_path_segments );
+        }
+        else {
+            $URI->path_segments( '' );
         }
 
-        if ( $segment eq '...' ) { # Should be treated as '.'
-            next;
-        }
+        my $new_url = $URI->as_string;
+        $new_url =  '/' . $new_url if ( $self->url =~ m,^/, );
+        $new_url =  $new_url . '/' if ( $self->url =~ m,/$, );
+        $new_url =~ s,^/+,/,;
 
-        push( @new_segments, $segment );
-    }
-
-    if ( @new_segments ) {
-        $URI->path_segments( @new_segments );
-    }
-    else {
-        $URI->path_segments( '' );
-    }
-
-    if ( $self->get_url =~ m,/$, ) {
-        $self->_set_url( $URI->as_string . '/' );
-    }
-    else {
-        $self->_set_url( $URI->as_string );
+        $self->_set_url( $new_url );
     }
 
     $self->make_canonical;
@@ -296,7 +283,7 @@ sub remove_dot_segments {
 
 =head2 remove_directory_index
 
-Removes well-known directory indexes, eg. "index.html", "default.asp" etc.
+Removes well-known directory indexes, eg. C<index.html>, C<default.asp> etc.
 
 Example:
 
@@ -306,57 +293,66 @@ Example:
 
     $normalizer->remove_directory_index;
 
-    print $normalizer->get_url; # http://www.example.com/?foo=/
+    print $normalizer->url; # http://www.example.com/?foo=/
 
-You are free to modify the global C<$DIRECTORY_INDEX_REGEXPS> arrayref to
-your own fitting:
+The default regular expressions for matching a directory index are:
 
-    $URL::Normalize::DIRECTORY_INDEX_REGEXPS = [ ... ];
+=over 4
 
-Default values are:
+    /default\.aspx?
+    /index\.cgi'
+    /index\.php\d?
+    /index\.pl,
+    /index\.s?html?
 
-    $DIRECTORY_INDEX_REGEXPS = [
-        '/default\.aspx?',
-        '/index\.cgi',
-        '/index\.php\d?',
-        '/index\.pl',
-        '/index\.s?html?',
-    ];
+=back
+
+You can override these by sending in your own list of regular expressions
+when creating the URL::Normalizer object:
+
+    my $normalizer = URL::Normalize->new(
+        url               => 'http://www.example.com/index.cgi?foo=/',
+        dir_index_regexps => [ 'MyDirIndex\.html' ], # etc.
+    );
+
+You can also choose to add regular expressions after the URL::Normalize
+object has been created:
+
+    my $normalizer = URL::Normalize->new(
+        url               => 'http://www.example.com/index.cgi?foo=/',
+        dir_index_regexps => [ 'MyDirIndex\.html' ], # etc.
+    );
+
+    # ...
+
+    push( @{$normalizer->dir_index_regexps}, 'MyDirIndex\.html' );
+
+Keep in mind that the regular expression are NOT case-sensitive, so the
+default C</default\.aspx?> expression will NOT match C</Default\.aspx?>.
 
 =cut
 
-our $DIRECTORY_INDEX_REGEXPS = [
-    '/default\.aspx?',
-    '/index\.cgi',
-    '/index\.php\d?',
-    '/index\.pl',
-    '/index\.s?html?',
-];
-
 sub remove_directory_index {
     my $self = shift;
 
-    my $URI  = $self->get_URI;
-    my $path = $URI->path;
+    if ( my $URI = $self->URI ) {
+        if ( my $path = $URI->path ) {
+            foreach my $regex ( @{$self->dir_index_regexps} ) {
+                $path =~ s,$regex,/,;
+            }
 
-    foreach my $regexp ( @{$DIRECTORY_INDEX_REGEXPS} ) {
-        $path =~ s,$regexp,/,;
+            $URI->path( $path );
+        }
+
+        $self->_set_url( $URI->as_string );
     }
-
-    $URI->path( $path );
-
-    #
-    # Set new 'url' value
-    #
-    return $self->_set_url( $URI->as_string );
-
 }
 
 =head2 sort_query_parameters
 
-Sorts the query parameters alphabetically.
+Sorts the URL's query parameters alphabetically.
 
-Uppercased parameters will be lowercased during sorting only, and if there are
+Uppercased parameters will be lowercased during sorting, and if there are
 multiple values for a parameter, the key/value-pairs will be sorted as well.
 
 Example:
@@ -367,48 +363,45 @@ Example:
 
     $normalizer->sort_query_parameters;
 
-    print $normalizer->get_url; # http://www.example.com/?a=0&A=1&b=2&c=3
+    print $normalizer->url; # http://www.example.com/?a=0&A=1&b=2&c=3
 
 =cut
 
 sub sort_query_parameters {
     my $self = shift;
 
-    my $URI = $self->get_URI;
+    if ( my $URI = $self->URI ) {
+        if ( $URI->as_string =~ m,\?, ) {
+            my $query_hash     = $URI->query_form_hash || {};
+            my $query_string   = '';
+            my %new_query_hash = ();
 
-    if ( $self->get_URI->as_string =~ m,\?, ) {
-        my $query_hash     = $URI->query_form_hash || {};
-        my $query_string   = '';
-        my %new_query_hash = ();
+            foreach my $key ( sort { lc($a) cmp lc($b) } keys %{$query_hash} ) {
+                my $values = $query_hash->{ $key };
+                unless ( ref $values ) {
+                    $values = [ $values ];
+                }
 
-        foreach my $key ( sort { lc($a) cmp lc($b) } keys %{$query_hash} ) {
-            my $values = $query_hash->{ $key };
-            unless ( ref $values ) {
-                $values = [ $values ];
-            }
-
-            foreach my $value ( @{$values} ) {
-                push( @{ $new_query_hash{lc($key)}->{$value} }, $key );
-            }
-        }
-
-        foreach my $sort_key ( sort keys %new_query_hash ) {
-            foreach my $value ( sort keys %{$new_query_hash{$sort_key}} ) {
-                foreach my $key ( @{$new_query_hash{$sort_key}->{$value}} ) {
-                    $query_string .= $key . '=' . $value . '&';
+                foreach my $value ( @{$values} ) {
+                    push( @{ $new_query_hash{lc($key)}->{$value} }, $key );
                 }
             }
+
+            foreach my $sort_key ( sort keys %new_query_hash ) {
+                foreach my $value ( sort keys %{$new_query_hash{$sort_key}} ) {
+                    foreach my $key ( @{$new_query_hash{$sort_key}->{$value}} ) {
+                        $query_string .= $key . '=' . $value . '&';
+                    }
+                }
+            }
+
+            $query_string =~ s,&$,,;
+
+            $URI->query( $query_string );
         }
 
-        $query_string =~ s,&$,,;
-
-        $URI->query( $query_string );
+        $self->_set_url( $URI->as_string );
     }
-
-    #
-    # Set new 'url' value
-    #
-    return $self->_set_url( $URI->as_string );
 }
 
 =head2 remove_duplicate_query_parameters
@@ -424,46 +417,45 @@ Example:
 
     $normalizer->remove_duplicate_query_parameters;
 
-    print $normalizer->get_url; # http://www.example.com/?a=1&a=2&b=3&c=4
+    print $normalizer->url; # http://www.example.com/?a=1&a=2&b=3&c=4
 
 =cut
 
 sub remove_duplicate_query_parameters {
     my $self = shift;
 
-    my $URI = $self->get_URI;
+    if ( my $URI = $self->URI ) {
+        my %seen      = ();
+        my @new_query = ();
 
-    my %seen      = ();
-    my @new_query = ();
+        foreach my $key ( $URI->query_param ) {
+            my @values = $URI->query_param( $key );
 
-    foreach my $key ( $URI->query_param ) {
-        my @values = $URI->query_param( $key );
-        foreach my $value ( @values ) {
-            unless ( $seen{$key}->{$value} ) {
-                push( @new_query, { key => $key, value => $value } );
-                $seen{$key}->{$value}++;
+            foreach my $value ( @values ) {
+                unless ( $seen{$key}->{$value} ) {
+                    push( @new_query, { key => $key, value => $value } );
+                    $seen{$key}->{$value}++;
+                }
             }
         }
+
+        my $query_string = '';
+        foreach ( @new_query ) {
+            $query_string .= $_->{key} . '=' . $_->{value} . '&';
+        }
+
+        $query_string =~ s,&$,,;
+
+        $URI->query( $query_string );
+
+        $self->_set_url( $URI->as_string );
     }
-
-    my $query_string = '';
-    foreach ( @new_query ) {
-        $query_string .= $_->{key} . '=' . $_->{value} . '&';
-    }
-
-    $query_string =~ s,&$,,;
-
-    $URI->query( $query_string );
-
-    #
-    # Set new 'url' value
-    #
-    return $self->_set_url( $URI->as_string );
 }
 
 =head2 remove_empty_query_parameters
 
-Removes empty query parameters, i.e. where there are keys with no value.
+Removes empty query parameters, i.e. where there are keys with no value. This
+only removes BLANK values, not values considered to be no value, like zero (0).
 
 Example:
 
@@ -471,30 +463,30 @@ Example:
         url => 'http://www.example.com/?a=1&b=&c=3',
     );
 
-    $Normalize->remove_empty_query_parameters;
+    $normalizer->remove_empty_query_parameters;
 
-    print $normalizer->get_url; # http://www.example.com/?a=1&c=3
+    print $normalizer->url; # http://www.example.com/?a=1&c=3
 
 =cut
 
 sub remove_empty_query_parameters {
     my $self = shift;
 
-    my $URI = $self->get_URI;
+    if ( my $URI = $self->URI ) {
+        foreach my $key ( $URI->query_param ) {
+            my @values = $URI->query_param( $key );
 
-    foreach my $key ( $URI->query_param ) {
-        my @values = $URI->query_param( $key );
+            $URI->query_param_delete( $key );
 
-        $URI->query_param_delete( $key );
-
-        foreach my $value ( @values ) {
-            if ( defined $value && length $value ) {
-                $URI->query_param_append( $key, $value );
+            foreach my $value ( @values ) {
+                if ( defined $value && length $value ) {
+                    $URI->query_param_append( $key, $value );
+                }
             }
         }
-    }
 
-    return $self->_set_url( $URI->as_string );
+        $self->_set_url( $URI->as_string );
+    }
 }
 
 =head2 remove_empty_query
@@ -509,20 +501,16 @@ Example:
 
     $normalizer->remove_empty_query;
 
-    print $Normalize->get_url; # http://www.example.com/foo
+    print $Normalize->url; # http://www.example.com/foo
 
 =cut
 
 sub remove_empty_query {
     my $self = shift;
 
-    my $url = $self->get_url;
-
+    my $url = $self->url;
     $url =~ s,\?$,,;
 
-    #
-    # Set new 'url' value
-    #
     $self->_set_url( $url );
 }
 
@@ -531,7 +519,7 @@ sub remove_empty_query {
 Removes the fragment from the URL, but only if they are at the end of the URL.
 
 For example "http://www.example.com/#foo" will be translated to
-"http://www.example.com/", but "http://www.example.com/#foo/bar" will stay the
+C<http://www.example.com/>, but C<http://www.example.com/#foo/bar> will stay the
 same.
 
 Example:
@@ -542,15 +530,19 @@ Example:
 
     $normalizer->remove_fragment;
 
-    print $normalizer->get_url; # http://www.example.com/bar.html
+    print $normalizer->url; # http://www.example.com/bar.html
 
 You should probably use this with caution, as most web frameworks today allows
 fragments for logic, for example:
 
+=over 4
+
     http://www.example.com/players#all
     http://www.example.com/players#banned
     http://www.example.com/players#top
 
+=back
+
 ...can all result in very different results, despite their "unfragmented" URL
 being the same.
 
@@ -559,7 +551,7 @@ being the same.
 sub remove_fragment {
     my $self = shift;
 
-    my $url = $self->get_url;
+    my $url = $self->url;
 
     $url =~ s{#(?:/|[^?/]*)$}{};
 
@@ -568,13 +560,17 @@ sub remove_fragment {
 
 =head2 remove_fragments
 
-Removes EVERYTHING after a '#'. You should use this sparsely, because a lot
-of web applications these days returns different output in response to what
-the fragment is, for example:
+Removes EVERYTHING after a C<#>. As with C<remove_fragment>, you should use this
+with caution, because a lot of web applications these days returns different
+output in response to what the fragment is, for example:
+
+=over 4
 
     http://www.example.com/users#list
     http://www.example.com/users#edit
 
+=back
+
 ...etc.
 
 =cut
@@ -601,138 +597,81 @@ Example:
 
     $normalizer->remove_duplicate_slashes;
 
-    print $normalizer->get_url; # http://www.example.com/foo/bar.html
+    print $normalizer->url; # http://www.example.com/foo/bar.html
 
 =cut
 
 sub remove_duplicate_slashes {
     my $self = shift;
 
-    my $URI  = $self->get_URI;
-    my $path = $URI->path;
+    if ( my $URI = $self->URI ) {
+        my $path = $URI->path;
 
-    $path =~ s,/+,/,g;
+        $path =~ s,/+,/,g;
 
-    $URI->path( $path );
+        $URI->path( $path );
 
-    #
-    # Set new 'url' value
-    #
-    $self->_set_url( $URI->as_string );
+        $self->_set_url( $URI->as_string );
+    }
 }
 
-=head2 remove_hostname_prefix
+=head2 remove_social_query_parameters
 
-Removes 'www' followed by a potential number before the actual hostname.
+Removes query parameters that are used for "social tracking."
+
+For example, a lot of newspapers posts links to their articles on Twitter,
+and adds a lot of (for us) "noise" in the URL so that they are able to
+track the number of users clicking on that specific URL. This method
+attempts to remove those query parameters.
 
 Example:
 
+    my $normalizer = URL::Normalize->new(
+        url => 'http://www.example.com/?utm_campaign=SomeCampaignId',
+    );
+
+    print $normalize->url; # 'http://www.example.com/'
+
+Default social query parameters are:
+
+=over 4
+
+    ncid
+    utm_campaign
+    utm_medium
+    utm_source
+
+=back
+
+You can override these default values when creating the URL::Normalize
+object:
+
+    my $normalizer = URL::Normalize->new(
+        url                 => 'http://www.example.com/',
+        social_query_params => [ 'your', 'list' ],
+    );
+
+You can also choose to add parameters after the URL::Normalize object
+has been created:
+
     my $normalizer = URL::Normalize->new(
         url => 'http://www.example.com/',
     );
 
-    $normalizer->remove_hostname_prefix;
-
-    print $normalizer->get_url; # http://example.com/
+    push( @{$normalizer->social_query_params}, 'QueryParam' );
 
 =cut
 
-sub remove_hostname_prefix {
-    my $self = shift;
-
-    my $URI  = $self->get_URI;
-    my $host = $URI->host;
-
-    #
-    # Count the number of parts in the hostname. If it's more than two parts
-    # in the URL, it's safe (...) to remove the "www\d*?\." prefix.
-    #
-    my @parts = split( /\./, $host );
-
-    if ( scalar(@parts) > 2 ) {
-        $host =~ s,^www\d*?\.,,;
-        $URI->host( $host );
-    }
-
-    #
-    # Set new 'url' value
-    #
-    $self->_set_url( $URI->as_string );
-}
-
-=head2 remove_social_query_parameters
-
-Removes query parameters that are used for "social tracking".
-
-For example, a lot of newspapers posts links to their articles on Twitter,
-and adds a lot of "noisee" in the URL so that they are able to track the
-number of users clicking on that specific URL. This method attempts to
-remove those query parameters.
-
-You are free to modify the global C<$SOCIAL_QUERY_PARAMETERS> arrayref to
-your own fitting:
-
-    $URL::Normalize::SOCIAL_QUERY_PARAMETERS = [ ... ];
-
-Default values are:
-
-    $SOCIAL_QUERY_PARAMETERS = [
-        'ncid',
-        'utm_campaign',
-        'utm_medium',
-        'utm_source',
-    ];
-
-=cut
-
-our $SOCIAL_QUERY_PARAMETERS = [
-    'ncid',
-    'utm_campaign',
-    'utm_medium',
-    'utm_source',
-];
-
 sub remove_social_query_parameters {
     my $self = shift;
 
-    my $URI = $self->get_URI;
+    if ( my $URI = $self->URI ) {
+        foreach ( @{$self->social_query_params} ) {
+            $URI->query_param_delete( $_ );
+        }
 
-    #
-    # Remove unwanted query parameters
-    #
-    foreach ( @{$SOCIAL_QUERY_PARAMETERS} ) {
-        $URI->query_param_delete( $_ );
+        $self->_set_url( $URI->as_string );
     }
-
-    #
-    # Set new 'url' value
-    #
-    $self->_set_url( $URI->as_string );
-
-}
-
-=head2 do_all
-
-Performs all of the normalization methods mentioned above.
-
-=cut
-
-sub do_all {
-    my $self = shift;
-
-    $self->make_canonical;
-    $self->remove_dot_segments;
-    $self->remove_directory_index;
-    $self->sort_query_parameters;
-    $self->remove_fragment;
-    $self->remove_duplicate_slashes;
-    $self->remove_duplicate_query_parameters;
-    $self->remove_empty_query_parameters;
-    $self->remove_hostname_prefix;
-    $self->remove_empty_query;
-    $self->remove_social_query_parameters;
-
-    return 1;
 }
 
 =head1 SEE ALSO
@@ -825,4 +764,6 @@ EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 =cut
 
+__PACKAGE__->meta->make_immutable;
+
 1; # End of URL::Normalize
